@@ -8,11 +8,12 @@ import os
 import pandas
 from pandas.errors import EmptyDataError
 import prometheus_client
+import string
 import random
 import warnings
 
 from .numerical import mean_stddev, get_numerical_ranges
-from .profile_types import identify_types
+from .profile_types import identify_types, determine_dataset_type
 from .spatial import nominatim_resolve_all, pair_latlong_columns, \
     get_spatial_ranges, parse_wkt_column
 from .temporal import get_temporal_resolution
@@ -65,6 +66,35 @@ def truncate_string(s, limit=140):
             return s[:space] + "..."
 
 
+DELIMITERS = set(string.punctuation) | set(string.whitespace)
+UPPER = set(string.ascii_uppercase)
+LOWER = set(string.ascii_lowercase)
+
+
+def expand_attribute_name(name):
+    name = name.replace('_', ' ').replace('-', ' ')
+
+    word = []
+    for c in name:
+        if c in DELIMITERS:
+            if word:
+                yield ''.join(word)
+                word = []
+            continue
+
+        if word:
+            if (
+                (word[-1] in string.digits) != (c in string.digits)
+                or (word[-1] in LOWER and c in UPPER)
+            ):
+                yield ''.join(word)
+                word = []
+
+        word.append(c)
+
+    yield ''.join(word)
+
+
 @PROM_PROFILE.time()
 def process_dataset(data, dataset_id=None, metadata=None,
                     lazo_client=None, nominatim=None, geo_data=None,
@@ -166,6 +196,7 @@ def process_dataset(data, dataset_id=None, metadata=None,
                 metadata['nb_rows'] = 0
                 metadata['nb_profiled_rows'] = 0
                 metadata['columns'] = []
+                metadata['types'] = []
                 return metadata
 
             logger.info("Dataframe loaded, %d rows, %d columns",
@@ -190,8 +221,12 @@ def process_dataset(data, dataset_id=None, metadata=None,
 
     if data.shape[0] == 0:
         logger.info("0 rows, returning early")
+        metadata['types'] = []
         return metadata
 
+    # Dataset types
+    dataset_types = set()
+
     # Lat / Long
     columns_lat = []
     columns_long = []
@@ -234,6 +269,12 @@ def process_dataset(data, dataset_id=None, metadata=None,
 
             structural_type, semantic_types_dict, additional_meta = \
                 identify_types(array, column_meta['name'], geo_data, manual)
+
+            # Identify a dataset type (numerical, categorial, spatial, or temporal)
+            dataset_type = determine_dataset_type(structural_type, semantic_types_dict)
+            if dataset_type:
+                dataset_types.add(dataset_type)
+
             # Set structural type
             column_meta['structural_type'] = structural_type
             # Add semantic types to the ones already present
@@ -590,6 +631,17 @@ def process_dataset(data, dataset_id=None, metadata=None,
 
         if spatial_coverage:
             metadata['spatial_coverage'] = spatial_coverage
+            if types.DATASET_SPATIAL not in dataset_types:
+                dataset_types.add(types.DATASET_SPATIAL)
+
+    # Attribute names
+    attribute_keywords = []
+    for col in columns:
+        attribute_keywords.append(col['name'])
+        kw = list(expand_attribute_name(col['name']))
+        if kw != [col['name']]:
+            attribute_keywords.extend(kw)
+    metadata['attribute_keywords'] = attribute_keywords
 
     # Sample data
     if include_sample:
@@ -604,6 +656,8 @@ def process_dataset(data, dataset_id=None, metadata=None,
         sample = sample.applymap(truncate_string)  # Truncate long values
         metadata['sample'] = sample.to_csv(index=False, line_terminator='\r\n')
 
+    metadata['types'] = sorted(dataset_types)
+
     # Return it -- it will be inserted into Elasticsearch, and published to the
     # feed and the waiting on-demand searches
     return metadata
