@@ -1,10 +1,13 @@
 import json
 import logging
+import numpy
+import numpy.random
 import prometheus_client
 import re
 import requests
 from sklearn.cluster import KMeans
 from sklearn.exceptions import ConvergenceWarning
+from sklearn.neighbors._kd_tree import KDTree
 import time
 from urllib.parse import urlencode
 
@@ -23,9 +26,10 @@ SPATIAL_RANGE_DELTA_LAT = 0.0001
 MAX_ADDRESS_LENGTH = 90  # 90 characters
 MAX_NOMINATIM_REQUESTS = 200
 NOMINATIM_BATCH_SIZE = 30
+NOMINATIM_MIN_SPLIT_BATCH_SIZE = 6  # Batches >=this are divided on failure
 
-LATITUDE = ('latitude', 'lat')
-LONGITUDE = ('longitude', 'long', 'lon', 'lng')
+LATITUDE = ('latitude', 'lat', 'ycoord', 'y_coord')
+LONGITUDE = ('longitude', 'long', 'lon', 'lng', 'xcoord', 'x_coord')
 
 
 PROM_NOMINATIM_REQS = prometheus_client.Counter(
@@ -109,18 +113,20 @@ def normalize_latlong_column_name(name, substrings):
 def pair_latlong_columns(columns_lat, columns_long):
     # Normalize latitude column names
     normalized_lat = {}
-    for i, (name, values_lat) in enumerate(columns_lat):
-        name = normalize_latlong_column_name(name, LATITUDE)
+    for i, (name, values_lat, annot_pair) in enumerate(columns_lat):
+        # check if a pair was defined by the user (human-in-the-loop)
+        name = annot_pair if annot_pair is not None else normalize_latlong_column_name(name, LATITUDE)
         normalized_lat[name] = i
 
     # Go over normalized longitude column names and try to match
     pairs = []
     missed_long = []
-    for name, values_long in columns_long:
-        norm_name = normalize_latlong_column_name(name, LONGITUDE)
+    for name, values_long, annot_pair in columns_long:
+        # check if a pair was defined by the user (human-in-the-loop)
+        norm_name = annot_pair if annot_pair is not None else normalize_latlong_column_name(name, LONGITUDE)
         if norm_name in normalized_lat:
             pairs.append((columns_lat[normalized_lat.pop(norm_name)],
-                          (name, values_long)))
+                          (name, values_long, annot_pair)))
         else:
             missed_long.append(name)
 
@@ -208,6 +214,40 @@ def nominatim_query(url, *, q):
         return res.json()
 
 
+def _nominatim_batch(url, batch, locations, cache):
+    try:
+        locs = nominatim_query(url, q=list(batch.keys()))
+    except requests.HTTPError as e:
+        if (
+            e.response.status_code == 500
+            and len(batch) >= max(2, NOMINATIM_MIN_SPLIT_BATCH_SIZE)
+        ):
+            # Try smaller batch size
+            batch_list = list(batch.items())
+            mid = len(batch) // 2
+            return (
+                _nominatim_batch(url, dict(batch_list[:mid]), locations, cache)
+                +
+                _nominatim_batch(url, dict(batch_list[mid:]), locations, cache)
+            )
+        raise
+
+    not_found = 0
+    for location, (value, count) in zip(locs, batch.items()):
+        if location:
+            loc = (
+                float(location[0]['lat']),
+                float(location[0]['lon']),
+            )
+            cache[value] = loc
+            locations.extend([loc] * count)
+        else:
+            cache[value] = None
+            not_found += count
+    batch.clear()
+    return not_found
+
+
 def nominatim_resolve_all(url, array, max_requests=MAX_NOMINATIM_REQUESTS):
     cache = {}
     locations = []
@@ -217,23 +257,6 @@ def nominatim_resolve_all(url, array, max_requests=MAX_NOMINATIM_REQUESTS):
     processed = 0
     batch = {}
 
-    def run_batch():
-        not_found_batch = 0
-        locs = nominatim_query(url, q=list(batch.keys()))
-        for location, count in zip(locs, batch.values()):
-            if location:
-                loc = (
-                    float(location[0]['lat']),
-                    float(location[0]['lon']),
-                )
-                cache[value] = loc
-                locations.extend([loc] * count)
-            else:
-                cache[value] = None
-                not_found_batch += count
-        batch.clear()
-        return not_found_batch
-
     for processed, value in enumerate(array):
         value = value.strip()
         if not value:
@@ -250,12 +273,12 @@ def nominatim_resolve_all(url, array, max_requests=MAX_NOMINATIM_REQUESTS):
         else:
             batch[value] = 1
             if len(batch) == NOMINATIM_BATCH_SIZE:
-                not_found += run_batch()
+                not_found += _nominatim_batch(url, batch, locations, cache)
                 if len(cache) >= max_requests:
                     break
 
     if batch and len(cache) < max_requests:
-        not_found += run_batch()
+        not_found += _nominatim_batch(url, batch, locations, cache)
 
     logger.info(
         "Performed %d Nominatim queries in %fs (%d hits). Found %d/%d",
@@ -266,3 +289,25 @@ def nominatim_resolve_all(url, array, max_requests=MAX_NOMINATIM_REQUESTS):
         processed,
     )
     return locations, non_empty
+
+
+def median_smallest_distance(points, tree=None):
+    """Median over all points of the distance to their closest neighbor.
+
+    This gives an idea of the "grid size" of a point dataset.
+    """
+    points = numpy.array(points)
+    if tree is None:
+        # points = numpy.unique(points, axis=0)  # Too slow
+        points = numpy.array(list(set(tuple(p) for p in points)))
+        tree = KDTree(points)
+
+    # Get the minimum distances to neighbors for a sample of points
+    rnd = numpy.random.RandomState(89)
+    sample_size = min(len(points), 100)
+    sample_idx = rnd.choice(len(points), sample_size, replace=False)
+    sample = points[sample_idx]
+    distances, _ = tree.query(sample, k=2, return_distance=True)
+
+    # Return the median of that
+    return numpy.median(distances[:, 1])
